package com.imari.boonobo.logic;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.Calendar;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import com.imari.boonobo.retrieve.ConnectionAction;

/**
 * @author Mar Llambí
 *	PFG - Event Recommender System
 *	Last updated: 18/01/2016
 */

public class RSystem {

	private UsersEvents ue;
	private int mode;
	//first Integer represents event, then user and predictions
	private Map <Integer, Map<Integer, Double>> predictions;
	
	//this method is the main one and represents the core of RS
	public void getPrediction(){
		
		predictions = new HashMap<Integer,  Map<Integer, Double> >();
		
		ue.calculateSimilarity(); 
		predictions = ue.obtainPrediction();
		insertPredictions();
	}
	
	//set mode attribute method
	public void setMode(int mode){
		this.mode = mode;
	}
	
	//get mode attribute method
	public int getMode(){
		return this.mode;
	}

	//this method prepares a new connection and set the final predictions between user-events
	private void insertPredictions(){
		Connection con;
		try{
			con = ConnectionAction.getConnection();
			// create a sql date object so we can use it in our INSERT statement
		     Calendar calendar = Calendar.getInstance();
		     java.sql.Date startDate = new java.sql.Date(calendar.getTime().getTime());
		      
		      
			String query = " insert into user_events (user_id, event_id, status_id, sr_score, created, modified)"
			        + " values (?, ?, ?, ?, ?, ?)";
			PreparedStatement preparedStmt;
			Iterator<Map.Entry<Integer, Map<Integer,Double>>> it = predictions.entrySet().iterator();
			 while(it.hasNext()){
				 Map.Entry<Integer, Map<Integer,Double>> pair = it.next();
				 Iterator<Map.Entry<Integer ,Double>> it_users = pair.getValue().entrySet().iterator();
				 while (it_users.hasNext()){
					 Map.Entry<Integer, Double> user_prediction = it_users.next();
					 preparedStmt = con.prepareStatement(query);
					 preparedStmt.setInt(1, user_prediction.getKey());
					 preparedStmt.setInt (2, pair.getKey());
					 preparedStmt.setInt (3, 6);
					 preparedStmt.setDouble (4, user_prediction.getValue());
					 preparedStmt.setDate (5, startDate);
					 preparedStmt.setDate (6, startDate);
					 preparedStmt.execute();
					 
				 }
			 }
			 con.close();
			
		}catch(Exception e){
			 System.err.println("Error inserting predictions on database");
		      e.printStackTrace();
		}
	}
	
	//this method sets the connection to DB and retrieves all necessary data
	public void retrieveData() throws SQLException{
		ue = new UsersEvents();
		Connection con;
		 try {
			  System.out.println("Loading data from database...");
		      con = ConnectionAction.getConnection();
		      String sql_total_users = "select max(id) from users";
		      String sql_users = "select * from users";
		      String sql_max_event_id = "select max(id) from events";
		      String sql_get_users_events = "select * from user_events where user_id = ?";
		      String sql_event_category = "select id, category_id from events";
		      String sql_user_preferences = "select user_id, categories from user_preferences where user_id = ?";
		      
		      //select total users
		      PreparedStatement pS = con.prepareStatement(sql_total_users);
		      ResultSet total_users = pS.executeQuery();
		      int count_users = 0;
		      while (total_users.next()) {
		    	  count_users = total_users.getInt("max(id)");
		      }
		      ue.setTotalUsers(count_users);
		      
		      //select total users id
		      pS = con.prepareStatement(sql_users);
		      ResultSet result_users = pS.executeQuery();
		      ue.setUsersId(result_users);
		      Integer users = ue.getTotal_users();
		      
		      //select user preferences on categories
		      pS = con.prepareStatement(sql_user_preferences);
		      for (int i = 0; i < users; i++){
		    	  pS.setInt(1, ue.getUserId(i));
		    	  ResultSet user_preferences = pS.executeQuery();
		    	  if (user_preferences.next()) {
				      ue.setUserPreferences(user_preferences); 
			      }
		      }
		      
		      //set max id events as total events
		      pS = con.prepareStatement(sql_max_event_id);
		      ResultSet total_events = pS.executeQuery();
		      int count_events = 0;
		      while (total_events.next()) {
		    	  count_events = total_events.getInt("max(id)");
		      }
		      ue.setTotalEvents(count_events);
		      
		      //now we need categories from events
		      pS = con.prepareStatement(sql_event_category);
		      ResultSet events_categories = pS.executeQuery();
		      ue.insertCategories(events_categories);
		      
		      ue.initializeMap();
		      
		      //now we have to retrieve all user-events statuses
		      pS = con.prepareStatement(sql_get_users_events);
		      for (int i = 0; i < users; i++){
		    	  pS.setInt(1, ue.getUserId(i));
		    	  ResultSet result_users_events = pS.executeQuery();
		    	  if (result_users_events.next()) {
			    	  ue.insertStatusStatement(result_users_events);   
			      }
		      }
		      
		      
			   // if mode is only friends, we also need them
			      if (this.mode == 1){
			    	  //ue.initializeFriends();
			    	  String sql_get_friends = "select * from user_users where user_id = ?";
				      pS = con.prepareStatement(sql_get_friends);
				      for (int i = 0; i < users; i++){
				    	  pS.setInt(1, ue.getUserId(i));
				    	  ResultSet result_users_friends = pS.executeQuery();
				    	  if (result_users_friends.next()) {
					    	  ue.insertFriends(result_users_friends);
					            
					      }
				      }
			      }
      
			con.close();
		      
		 }catch (Exception e) {
			  System.err.println("Error loading data from database");
		      e.printStackTrace();
		 }
	}
}
