package com.imari.boonobo.logic;

import java.util.ArrayList;

import java.util.Arrays;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

/**
 * @author Mar Llambí
 *	PFG - Event Recommender System
 *	Last updated: 18/01/2016
 */

public class User {
	
	private int id;
	private List<Integer> friends_id;
	private Map<Integer,Double> user_status;
	private Map<Integer,Double > users_similarity;
	private Double average;
	private List<String> preferences;
	private boolean pref_set;
	
	//constructor by id
	public User(int id){
		this.id = id;
		average = -1.0;
		pref_set = false;
		preferences = new ArrayList<String> (14); //14 is the total num of categories we have
	}
	//get id attribute method
	public int getId (){
		return id;
	}
	
	//set user_status attribute method
	public void initStatus(Map<Integer,Double> u_s){
		user_status = new HashMap<Integer,Double>(u_s);
	}
	//set friends_id attribute method
	public void setFriends(List<Integer> friends){
		friends_id = new ArrayList<Integer> (friends);
	}
	
	//get user_status attribute method
	public Map<Integer,Double> getStatus(){
		return user_status;
	}
	
	//get friends_id attribute method
	public List<Integer> getFriends(){
		return friends_id;
	}
	
	//set users_similarity attribute method
	public void initSimilarity(int total_users){
		users_similarity = new HashMap<Integer,Double>(total_users+1);
	}
	
	//this method set a similarity between a friend and this user
	public void putSimilarity(int friend, Double correlation){
		users_similarity.put(friend, correlation);
	}
	
	//get average attribute method
	public Double getAverage(){
		return average;
	}
	
	//set average attribute method
	public void setAverage (Double avg){
		average = avg;
	}
	
	//to know if exists a relation between event and user
	public boolean existsRelation(Integer event){
		if (user_status.get(event) != 0.0){
			return true;
		}else{
			return false;
		}
	}
	
	//get the size of users_similarity method
	public int SimilaritySize(){
		return users_similarity.size();
	}
	
	//get users_similarity attribute method
	public Map<Integer,Double > getSimilarities(){
		return users_similarity;
	}
	
	//get status user-event
	public Double getStatusForEvent(int event){
		return user_status.get(event);
	}
	
	//insert new Status values on user
	public void copyStatus(Map<Integer,Double> status){
		Iterator<Map.Entry<Integer, Double>> it = status.entrySet().iterator();
		while(it.hasNext()){
        	Map.Entry<Integer,Double> pair = it.next();
        	user_status.put(pair.getKey(), pair.getValue());
        }
		
		
	}
	
	//set user preferences method
	public void setPreferences(String preferences){
		//we remove chars "{}"
		preferences = preferences.substring(1);
		preferences = preferences.substring(0, preferences.length()-1);
		
		//now we need to split it to get the categories id
		if(preferences.contains(",")){
			String [] auxiliar = preferences.split(",");
			this.preferences = Arrays.asList(auxiliar);
			pref_set = true;
		}
			
	}
	
	//indicates if user have preferences list
	public boolean havePreferences(){
		return pref_set;
	}
	
	//indicates if this user have preference category
	public boolean havePreference(int category){
		return preferences.contains(Integer.toString(category));
	}

}
