package com.imari.boonobo.logic;

import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.apache.commons.math3.stat.correlation.PearsonsCorrelation;
import org.apache.commons.math3.stat.descriptive.DescriptiveStatistics;
import org.apache.commons.math3.stat.descriptive.summary.Sum;

/**
 * @author Mar Llambí
 *	PFG - Event Recommender System
 *	Last updated: 18/01/2016
 */

public class UsersEvents {

	private int total_users; //remember that user_id in db starts with id=1
	private int total_events; //remmeber that always need +1 to access the last position
	

	//private List<Integer> events_id;
	private int mode;
	private Map<Integer,User> list_users;
	private Map<Integer,Event> list_events;
	
	//default constructor
	public UsersEvents(){
		this.total_users = 0;
		this.total_events = 0;
		this.mode = 0;
	}
	
	//set total_users attribute method
	public void setTotalUsers(int users){
		this.total_users = users;
	}
	
	//set total_events attribute method
	public void setTotalEvents(int events){
		this.total_events = events;
	}
	
	//get total_events attribute method
	public int getTotal_events() {
		return this.total_events;
	}
	
	//get total_users attribute method
	public int getTotal_users() {
		return this.total_users;
	}


	//private method to get the right meaning of user-event status
	private double translateStatus (Integer status){
		if (status == 1){
			return 1.0;
		}else if (status == 2){
			return 3.0;
		}else if (status == 3){
			return 2.0;
		}else if (status == 4){
			return -1.0;
		}else if (status == 5){
			return -2.0;
		}else{
			return 5.0;
		}
	}

	//method to initialize some necessary maps
	public void initializeMap(){
		
		this.mode = 0; //default all users
		Map<Integer,Double> event_status = new HashMap<Integer,Double>(total_events+1);
		for (int j = 0; j <= this.total_events; j++){
			if (list_events.containsKey(j)){
				event_status.put(j,0.0);
			}
		}
		for(int i = 0; i <= this.total_users; i++ ){
			if(list_users.containsKey(i)){
				list_users.get(i).initStatus(event_status);
				list_users.get(i).setAverage(-1.0);
			}
		}
		
	}
	
	//method to set user-events relation with status values from database ResultSet
	public boolean insertStatusStatement (ResultSet user_event_status ){
		try {

			Map<Integer,Double> event_status = new HashMap<Integer,Double>(total_events+1);
			for (int j = 0; j <= this.total_events; j++){
				if (list_events.containsKey(j)){
					event_status.put(j,0.0);
				}

			}
			Integer user_id = user_event_status.getInt("user_id");
			if (user_id != null){
				Integer event_id = user_event_status.getInt("event_id");
				Integer status = user_event_status.getInt("status_id");  
				if (event_id != null){
					event_status.put(event_id, translateStatus(status));
				}
				while (user_event_status.next()){
					event_id = user_event_status.getInt("event_id");
					status = user_event_status.getInt("status_id");  
					event_status.put(event_id, translateStatus(status));
				}
				list_users.get(user_id).copyStatus(event_status);
				return true;
			}else{
				System.out.println("Error: ResultSet from user_events is empty!");
				return false;
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}
	}
	
	//method to set user-user relation values from database ResultSet
	public boolean insertFriends (ResultSet result_users_friends ){
		try {
			this.mode = 1; //now only wants friends
			List<Integer> user_friends_aux = new ArrayList<Integer> ();
			Integer user_id = result_users_friends.getInt("user_id");
			if (user_id != 0){
				Integer friend_id = result_users_friends.getInt("friend_user_id");
				if (friend_id != null){
					user_friends_aux.add(friend_id);
				}
				while (result_users_friends.next()){
					friend_id = result_users_friends.getInt("friend_user_id"); 
					user_friends_aux.add(friend_id);
				}
				list_users.get(user_id).setFriends(user_friends_aux);
				return true;
			}else{
				System.out.println("Error: ResultSet from user_friends is empty!");
				return false;
			}
			
		} catch (SQLException e) {
			e.printStackTrace();
			return false;
		}
	}
	
	//this method converts Status Map to ArrayList
	private double[] StatusMaptoArray(Integer user_id){

		double[] array = new double[list_users.get(user_id).getStatus().size()+1];
		Iterator<Map.Entry<Integer, Double>> it = list_users.get(user_id).getStatus().entrySet().iterator();

        int i = 0;
        while(it.hasNext()){
        	Map.Entry<Integer,Double> pair = it.next();
        	array[i] = pair.getValue();
            i++;
        }
		return array;
	}
	
	//this method calculate similarity between users (depen of the mode) by Pearson Correlation
	public void calculateSimilarity(){
		
		try{
			System.out.println("Let's calculate Similarity between users");
			double pearson_calculate = 0.0;
			for (int i = 0; i < total_users; i++){
				if(list_users.containsKey(i)){
				list_users.get(i).initSimilarity(total_users);
					if (this.mode == 1) { //if mode is only friends, we have to check if they are
						Iterator<Integer> it = list_users.get(i).getFriends().iterator();
						while (it.hasNext()) {
							Integer friend_id = it.next();
							// if they are friends
							pearson_calculate = new PearsonsCorrelation().correlation(StatusMaptoArray(i),StatusMaptoArray(friend_id));
							if (Double.isNaN(pearson_calculate)) {
								pearson_calculate = 0.0;
							}
							list_users.get(i).putSimilarity(friend_id, pearson_calculate);
							 System.out.println("Pearson correlation between user "+i
									 +" with user "+friend_id+ " is "+pearson_calculate);
						}
					}else{ //the mode is all users with every user
						Iterator<Map.Entry<Integer,User>> it = list_users.entrySet().iterator();
						while (it.hasNext()) {
							Map.Entry<Integer, User> pair = it.next();
							Integer user_id = pair.getKey();
							if (user_id != i){//we don't want to calculate similarity if the user is the same
								pearson_calculate = new PearsonsCorrelation().correlation(StatusMaptoArray(user_id),
										StatusMaptoArray(i));
								if (Double.isNaN(pearson_calculate)) {
									pearson_calculate = 0.0;
								}
								list_users.get(i).putSimilarity(user_id, pearson_calculate);
								//user_i_similarities.put(pair.getKey(), pearson_calculate);
								// System.out.println("Pearson correlation between user "+i
										// +" with user "+user_id+ " is "+pearson_calculate);
	
							}
						}
					}
				}
			}
		}catch(Exception e){
			System.err.println(e.getMessage());
			e.printStackTrace();
		}
		
		
	}
	
	//this method converts user similarities Map to ArrayList
	private double[] MaptoArray(Integer user_id){
		double[] array = new double[list_users.get(user_id).SimilaritySize()];
		Iterator<Map.Entry<Integer, Double>> it = list_users.get(user_id).getSimilarities().entrySet().iterator();
        int i = 0;
        while(it.hasNext()){
        	Map.Entry<Integer,Double> pair = it.next();
        	array[i] = pair.getValue();
            i++;
        }
		return array;
	}
	
	//getAverage method from user (calculate a new one or get the existing one)
	private double getAverage (Integer user_id){
	
		if (list_users.get(user_id).getAverage() != -1.0){
				return list_users.get(user_id).getAverage();
		}else{
			double user_a_average = new DescriptiveStatistics (StatusMaptoArray(user_id)).getMean();
			System.out.println ("Status average of user "+user_id+" is "+user_a_average);
			list_users.get(user_id).setAverage(user_a_average);
			return user_a_average;
		}
				
	}
	
	//this method obtains the similarity between a rating from user and its other ratings
	private double[] ArraySimilarityBetweenRatings (Integer user_id, Integer event_id){
		
		double[] array = new double[list_users.get(user_id).SimilaritySize()];
		try{
		
			Iterator<Map.Entry<Integer, Double>> it = list_users.get(user_id).getSimilarities().entrySet().iterator();
			double result = 0.0;
	        int i = 0;
	        while(it.hasNext()){
	        	Map.Entry<Integer,Double> pair = it.next();
	        	double user_a_average = getAverage(pair.getKey());
	        	double rating_user_it_event = list_users.get(pair.getKey()).getStatusForEvent(event_id);
	        	result = pair.getValue() * (rating_user_it_event-user_a_average);
	        	array[i] = result;
	            i++;
	        }       
			return array;
		}catch( Exception e){
				System.err.println(e.getMessage());
				e.printStackTrace();
				return array;
		}

	}
	
	//This method represents content filtering and weights the prediction from preferences
	private Double ContentFiltering(Double prediction, Integer event, Integer user){
		int category = list_events.get(event).getCategory();
		if(list_users.get(user).havePreferences()){
			if(list_users.get(user).havePreference(category)){
				return  prediction+0.5;
			}else{
				return prediction-0.5;
				
			}
		}else{
			return prediction;
		}
	}
	
	//this method includes all the process to obtain predictions for every user
	public Map <Integer, Map<Integer, Double>> obtainPrediction (){
		
		Map <Integer, Map<Integer,Double>> predictions = new HashMap <Integer, Map<Integer,Double>>();
		try{
			Integer event_id = 0;
			Iterator<Map.Entry<Integer, Event>> eventIterator = list_events.entrySet().iterator();
			while (eventIterator.hasNext()) {
				Map.Entry<Integer,Event> pair = eventIterator.next();
				event_id = pair.getKey();
				Map<Integer,Double> user_predictions = new HashMap <Integer,Double>();
				for (int i = 0; i< total_users; i++){
					if(list_users.containsKey(i)){
						if (!list_users.get(i).existsRelation(event_id)){
							double user_a_average = getAverage(i);
							Sum summatory_similarity = new Sum(); //represents summatory sim(a,b)
							Sum summatory_similarity_ratings = new Sum(); //represents summatory sim(a,b) * (rating b-p - mean ratings b)
							
							double summatory_similarity_ratings_total = summatory_similarity_ratings.evaluate(ArraySimilarityBetweenRatings(i,event_id));
							double summatory_total = summatory_similarity.evaluate(MaptoArray(i));
							
					
							double prediction_a_p = ((user_a_average)+(1/summatory_total))* summatory_similarity_ratings_total;
							if (Double.isNaN(prediction_a_p)){
								prediction_a_p = -1.0;
							}
							prediction_a_p = ContentFiltering(prediction_a_p, event_id, i);
							if (prediction_a_p >= 1.5){
								user_predictions.put(i, prediction_a_p);
								System.out.println ("Prediction for user "+i+" for event "+event_id+" is "+prediction_a_p);
							}else{
								System.out.println ("Prediction for user "+i+" for event "+event_id+" is "+prediction_a_p+" too low to consider");
	
							}
						}else{
							System.out.println ("Notice: Already exists relationship between user "+i+" and event "+event_id);
						}
					}
				}
				if (!user_predictions.isEmpty()){
					predictions.put(event_id, user_predictions);
				}

			}
			return predictions;
			
		}catch( Exception e){
			
			System.err.println(e.getMessage());
			e.printStackTrace();
			return predictions;
		}
		
	}
	
	
	//this method set users id from database Resultset
	public void setUsersId(ResultSet result_users){
		int user_id;
		try{
			list_users = new HashMap <Integer,User>(total_users+1);
			while (result_users.next()) {
		          user_id = result_users.getInt("id");
		          User u = new User(user_id);
		    	  list_users.put(user_id,u); 
		    }
		}
		catch(Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
		
	}
	
	//this method set categories to events from database Resultset
	public void insertCategories (ResultSet events_categories){
		try{
			int id = 0;
			int category = 0;
			list_events = new HashMap <Integer,Event>(total_events+1);
			while (events_categories.next()) {
		          id = events_categories.getInt("id");
		          category = events_categories.getInt("category_id");
		          Event e = new Event(id,category);
		          list_events.put(id, e);
		    }
		}
		catch(Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	//this method insert preferences to user from database resultset
	public void setUserPreferences (ResultSet user_preferences){
		try{
			int id;
			String preferences;
			while(user_preferences.next()){
				id = user_preferences.getInt("user_id");
				preferences = user_preferences.getString("categories");
				list_users.get(id).setPreferences(preferences);
			}
		}
		catch(Exception e){
			System.out.println(e.getMessage());
			e.printStackTrace();
		}
	}
	
	//this method returns user i if exists, else return 0
	public int getUserId(int id){
		if (list_users.containsKey(id)){
			return id;
		}else return 0;
	}
	
	
}
