<?php
/**
 * Application level Controller
 *
 * This file is application-wide controller file. You can put all
 * application-wide controller-related methods here.
 *
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link          http://cakephp.org CakePHP(tm) Project
 * @package       app.Controller
 * @since         CakePHP(tm) v 0.2.9
 * @license       http://www.opensource.org/licenses/mit-license.php MIT License
 */

App::uses('Controller', 'Controller');
App::import('Config','Constants');
App::uses('DateUtilities','Lib');

/**
 * Application Controller
 *
 * Add your application-wide methods in the class below, your controllers
 * will inherit them.
 *
 * @package		app.Controller
 * @link		http://book.cakephp.org/2.0/en/controllers.html#the-app-controller
 */
class AppController extends Controller {
	
	public $components = array('Cookie','Session','DebugKit.Toolbar',
			'Auth' => array(
					'loginRedirect' => array('controller' => 'pages', 'action' => 'index'),
					'logoutRedirect' => array('controller' => 'users', 'action' => 'login'),
					'authorize' => array('Controller'),
					'authError' => 'No pot accedir a aquesta pàgina'
			),
			'Security' => array(
					'csrfExpires' => '+1 hour',
					'csrfUseOnce' => false,
					'blackHoleCallback' => 'blackhole'
			),
			'AjaxException','Utilities','LogUtil'
	);
	
	
	public $helpers = array( 'Html' => array('className' => 'Utilities'), 'Js' => array('Jquery'), 'JqueryValidation','Notification.Notification', 'GoogleMap');
	public $uses = array('User','Chat','Message', 'Category');
	
	public function afterFilter() {
	    $this->LogUtil->log();
	}
	
	function _setErrorLayout() {
		if ($this->name == 'CakeError') {
			$this->layout = 'error';
		}
	}
	
	function beforeRender () {
		$this->_setErrorLayout();
	}

	
	public function beforeFilter() {

		if($this->Auth->loggedIn()){
			$user = $this->Auth->user();
			
			//setting the timezone for all dates using TimeHelper
			//Configure::write('Config.timezone', $this->Session->read('Auth.User.timezone'));
			
			$categories = $this->Category->find('all');
			$this->set('categories', $categories);
			$this->set('current_user',$user); // Creem una variable amb el usuari actual    		
    		$this->set('chat_users',$this->User->find('all',array('conditions' => array('id <>' => $user['id']))));
    		$chat = $this->Chat->find('list',array('conditions' => array("OR"=> array('user_id' =>  $user['id'],'reciever_user_id' => $user['id']))));

    		$newmessages = null;
    		$this->set('new_messages',$newmessages);
    		
    		//User last connection update 
            $user['last_connection'] = date(DATE_ATOM);
            $this->User->updateAll(array("User.last_connection" => "'".date(DATE_ATOM)."'"),array('User.id' => $user['id']));
            
		}
	}
	
	
	public function isAuthorized($user) {
		return true;
	}
	
	
	public function blackhole($type) {
		if($this->request->is('ajax')) {
			$this->Session->delete('Message.flash');
			$this->AjaxException->throwException(400,'ERROR',__('Alguna cosa ha anat malament'));
		}
		else{
			switch ($type) {
				case 'auth':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de validació de formulari, o un error de manca de coincidència controlador/acció'),"flash_fail");
					break;
	
				case 'csrf':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de CSRF'),"flash_fail");
					break;
	
				case 'get':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de restricció de HTTP'),"flash_fail");
					break;
	
				case 'post':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de restricció de HTTP'),"flash_fail");
					break;
	
				case 'put':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de restricció de HTTP'),"flash_fail");
					break;
	
				case 'delete':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de restricció de HTTP'),"flash_fail");
					break;
	
				case 'secure':
					$this->Session->setFlash(__('Problema de seguretat: indica un error de restricció de SSL'),"flash_fail");
					break;
	
				default:
					$this->Session->setFlash(__('Problema de seguretat'),"flash_fail");
					break;
			}
				
			//$this->redirect(array('controller'=>$this->request->controller,'action' => $this->request->action));
			//$this->redirect('/'.$this->request->url);
				
			throw new BadRequestException();
		}
	}
	
}
