<?php
App::uses('AppController', 'Controller');
App::uses('CakeTime', 'Utility');
App::uses('DateUtilities', 'Lib');

/**
 * Events Controller
 *
 * @property PaginatorComponent $Paginator
*/
class EventsController extends AppController {

	/**
	 * Components
	 *
	 * @var array
	 */
	public $components = array('Paginator');
	public $uses = array('Event', 'EventsComment', 'UserEvent', 'User');


	public function beforeFilter() {
		parent::beforeFilter();

		$this->__securityConf();
	}


	/**************** SECURITY FUNCTIONS ****************/

	private function __securityConf(){
		$this->Security->unlockedActions = array('mainPage', 'send_comment', 'viewEvent', 'changeStatus', 'eventRead', 'topTwenty', 'allEvents', 'noMore');
	}


	public function isAuthorized($user) {

		$action = $this->request->action;
			
		if(in_array($action, array("assignUser", "send_publication"))){
			if($user['role_id'] != Constants::$ROLE_ADMIN){
				if($this->request->is('ajax')) {
					$this->AjaxException->throwException(401,'ERROR','Unauthorized');
				}
				$this->Session->setFlash(__('No tens permissos per realitzar aquesta acció'),'flash_fail');
				return false;
			}
		}


		return true;
	}

	/************** END SECURITY FUNCTIONS **************/


	public function mainPage() {
		$this->set('title_for_layout', __('Gestió dels events'));
		$this->set('page', 'main_page');
		$this->initializePage('mainpage');

	}
	
	public function topTwenty(){
		$this->set('title_for_layout', __('Gestió dels events'));
		$this->set('page', 'top_twenty');
		$this->initializePage('toptwenty');
		
	}
	public function random(){
		$this->set('title_for_layout', __('Gestió dels events'));
		$this->set('page', 'random');
		$this->initializePage('random');
	
	}
	public function allEvents(){
		$this->set('title_for_layout', __('Gestió dels events'));
		$this->set('page', 'all_events');
		$this->initializePage('allevents');
	
	}
	
	private function initializePage($view){
		$current_user = $this->Auth->user();
		if (isset($this->request->data['selectOptions']['category_filter'])){
			$category = $this->request->data['selectOptions']['category_filter'];
		}else{
			$category = Constants::$CATEGORY_ALL;
		}
		if (isset($this->request->data['selectOptions']['time_type']) && $this->request->data['selectOptions']['time_type'] != Constants::$FILTER_ALL_TIME ){
			$date = $this->request->data['selectOptions']['time_filter'];
		}else{
			$date = '';
		}
		
		if($category == Constants::$CATEGORY_ALL){
			$filter_category = array();
		}else{
			$filter_category = array('category_id' => $category);
		}
		
		$start_date = date('Y-m-d H:i:s');
		if ($date == ''){
			$filter_time = array('or' => array('start_time >=' => $start_date, 'end_time >= '=> $start_date));
		}else{
			$final_time = date('Y-m-d H:i:s', strtotime($date));
			$filter_time = array(
					'or' => array(
							'and' => array(
									'start_time >= ' => $start_date,
									'start_time <= ' => $final_time),
							'and' => array(
									'end_time >= ' => $start_date,
									'end_time <= ' => $final_time)
					)
			);
		}
		$elementsPerPage = Constants::$ELEMENTS_PER_PAGE;
		$offset = 0;
		if ($view == 'mainpage'){
			$events = $this->Event->findRecommendedEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage, $current_user['id']);
		}elseif ($view == 'toptwenty'){
			$events = $this->Event->findTopEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage);
			
		}elseif($view == 'allevents'){
			$events = $this->Event->findEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage);
			
		}elseif($view == 'random'){
			$events = $this->Event->findRandomEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage);
			
		}
		
		$offset = 0 + count($events);
		$this->set('events', $events);
		$this->set('filter_category_set',$category );
		$this->set('offset', $offset);
		if (isset($this->request->data['selectOptions']['time_type'])){
			$this->set('filter_time_set',$this->request->data['selectOptions']['time_type']);
		}else{
			$this->set('filter_time_set',Constants::$FILTER_ALL_TIME );
		}
		return $events;
	}
	
	public function getEvents($offset = null, $elementsPerPage = null, $category = null, $time_filter = null, $date_ajax = null) {
		$this->set('title_for_layout', __('Gestió dels events'));
		$this->set('page', 'main_page');
		$current_user = $this->Auth->user();
		if($this->request->is("ajax")){
			$this->layout = false;
			if($category == Constants::$CATEGORY_ALL){
				$filter_category = array();
			}else{
				$filter_category = array('category_id' => $category);
			}
			if ($time_filter != Constants::$FILTER_ALL_TIME ){
				$date = $date_ajax;
			}else{
				$date = '';
			}
			if (!isset($this->request->query['view'])){
				$view = 'allevents';
			}else{
				$view = $this->request->query['view'];
			}
			$start_date = date('Y-m-d H:i:s');
			if ($date == ''){
				$filter_time = array('or' => array('start_time >=' => $start_date, 'end_time >= '=> $start_date));
			}else{
				$final_time = date('Y-m-d H:i:s', strtotime($date));
				$filter_time = array(
						'or' => array(
								'and' => array(
										'start_time >= ' => $start_date,
										'start_time <= ' => $final_time),
								'and' => array(
										'end_time >= ' => $start_date,
										'end_time <= ' => $final_time)
						)
				);
			}
			$elementsPerPage = Constants::$ELEMENTS_PER_PAGE;
			if ($view == 'mainpage'){
				$events = $this->Event->findRecommendedEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage, $current_user['id']);
			}elseif($view == 'allevents'){
				$events = $this->Event->findEventsCategoryTime($filter_time, $filter_category, $offset, $elementsPerPage);
					
			}
			$offset = 0 + count($events);
			$this->set('events', $events);
			$this->set('offset', $offset);
			$this->render('/Elements/Events/list_events', false);
		}
		
	}

	public function viewEvent($id = null) {
		$current_user = $this->Auth->user();
		if($this->Event->exists($id)){
			$event = $this->Event->findEventAttributes($id);
			if (isset($event[0]['Event']['start_time']) && $event[0]['Event']['start_time'] != '' ){
				$start_time = date('d-m-Y H:i', strtotime($event[0]['Event']['start_time']));
			}else{
				$start_time = __("No especificat");
			}
			$this->set('start_time',$start_time);
			if (isset($event[0]['Event']['end_time']) && $event[0]['Event']['end_time'] != '' ){
				$end_time = date('d-m-Y H:i', strtotime($event[0]['Event']['end_time']));
			}else{
				$end_time = __("No especificat");
			}
			$this->set('end_time',$end_time);
				
			$image = $id % 19;
				
			$this->EventsComment->contain('User');
			$events_comments = $this->EventsComment->findAllByEventId($id);
			$this->set('events_comments', $events_comments);
				
			$this->UserEvent->contain('User');
			$event_assistents = $this->UserEvent->find('all', array( 
					'conditions' => array(
					'event_id' => $id,
					'status_id' => Constants::$USER_EVENT_GO )));
			$this->set('event_assistents', $event_assistents);
			
			
			$user_event = $this->UserEvent->find('first', array(
					'conditions' => array('user_id' => $current_user['id'], 'event_id' => $id)));
			if ($user_event['UserEvent']['id'] != null){
				$this->set('event_user_status', $user_event['UserEvent']['status_id']);
			}
				
			$this->set('image_id', $image);
			$this->set('title_for_layout', $event[0]['Event']['title']);
			$this->set('page', 'view_event');
			$this->set('event', $event);
		}else{
			$this->set('title_for_layout', __('Event no trobat'));
			$this->set('page', 'view_event');
			$this->Session->setFlash(__("No s'ha trobat l'event especificat"),'flash_fail');

		}
	}


	/**
	 * linear regression function
	 * @param $x array x-coords
	 * @param $y array y-coords
	 * @returns array() m=>slope, b=>intercept
	 */
	function linear_regression($x, $y) {
	  
	  
		//CakeLog::debug($separado_por_comas = implode(",", $x));
		//CakeLog::debug($separado_por_comas = implode(",", $y));

		// calculate number points
		$n = count($x);
	  
		foreach($x as $key => $value){
			$x[$key] = $value / $n;
		}
	  
		// ensure both arrays of points are the same size
		if ($n != count($y)) {
			trigger_error("linear_regression(): Number of elements in coordinate arrays do not match.", E_USER_ERROR);
		}

		// calculate sums
		$x_sum = array_sum($x);
		$y_sum = array_sum($y);

		$xx_sum = 0;
		$xy_sum = 0;

		for($i = 0; $i < $n; $i++) {
			$xy_sum+=($x[$i]*$y[$i]);
			$xx_sum+=($x[$i]*$x[$i]);
		}

		// calculate slope
		$m = (($n * $xy_sum) - ($x_sum * $y_sum)) / (($n * $xx_sum) - ($x_sum * $x_sum));

		// calculate intercept
		$b = ($y_sum - ($m * $x_sum)) / $n;

		// return result
		return array("m"=>$m, "b"=>$b);
	}



	//no borrar, m'anira be x mirar el tema peticions
	public function submitTraining($message_id = null, $message_type = null, $message_mood = null, $message_category = null){
		$this->autoRender = false;
			
		if(empty($message_id)){
			$message_id = $this->request->data['message_id'];
		}
		if(empty($message_type)){
			$message_type = $this->request->data['message_type'];
		}
		if(empty($message_mood)){
			$message_mood = $this->request->data['message_mood'];
		}
		if(empty($message_category)){
			$message_category = $this->request->data['message_category'];
		}
			
		if($this->request->is("post")){

			$url = Constants::$WS_RECATEGORIZER;

			if($message_type == 'twitter') {
				if (!$this->Tweet->exists($message_id)) {
					throw new NotFoundException(__('Tweet no vàlid'));
				}

				$message = $this->Tweet->findById($message_id);
				$message['Tweet']['mood'] = $message_mood;
				$message['Tweet']['category'] = $message_category;
				$this->Tweet->save($message);

				//Fix for Webservice
				$text = str_replace('"', '', trim(preg_replace('/\s+/', ' ', $message['Tweet']['text'])));
					
				$url .= "text=".urlencode($text)."&polarity=".$message_mood."&Category=".$message_category;
				$result = $this->Utilities->webservicePost($url);

				$result = json_decode($result,true);

				if($result['status'] != Constants::$WS_OK){
					CakeLog::error('submitTraining');
				}

				if($message['Tweet']['top_parent_reply_to_status_id'] === null) {
					return $this->reloadTweet($message_id);
				}
				else {
					return $this->reloadTweet($message['Tweet']['top_parent_reply_to_status_id']);
				}

			} else if($message_type == 'facebook_post') {
				if (!$this->Post->exists($message_id)) {
					throw new NotFoundException(__('Publicació no vàlida'));
				}

				$message = $this->Post->findById($message_id);
				$message['Post']['mood'] = $message_mood;
				$message['Post']['category'] = $message_category;
				$this->Post->save($message);

				//Fix for Webservice
				$text = str_replace('"', '', trim(preg_replace('/\s+/', ' ', $message['Post']['message'])));
					
				$url .= "text=".urlencode($text)."&polarity=".$message_mood."&Category=".$message_category;
				$result = $this->Utilities->webservicePost($url);

				$result = json_decode($result,true);

				if($result['status'] != Constants::$WS_OK){
					CakeLog::error('submitTraining');
				}
					
				return $this->reloadPost($message_id);

			} else if($message_type == 'facebook_comment'){
				if (!$this->Comment->exists($message_id)) {
					throw new NotFoundException(__('Comentari no vàlid'));
				}

				$message = $this->Comment->findById($message_id);
				$message['Comment']['mood'] = $message_mood;
				$message['Comment']['category'] = $message_category;
				$this->Comment->save($message);

				//Fix for Webservice
				$text = str_replace('"', '', $message['Comment']['message']);
					
				$url .= "text=".urlencode($text)."&polarity=".$message_mood."&Category=".$message_category;
				$result = $this->Utilities->webservicePost($url);

				$result = json_decode($result,true);

				if($result['status'] != Constants::$WS_OK){
					CakeLog::error('submitTraining');
				}
					
				return $this->reloadPost($message['Comment']['post_id']);
			} else if($message_type == 'facebook_thread'){
				if (!$this->Thread->exists($message_id)) {
					throw new NotFoundException(__('Comentari no vàlid'));
				}
					
				$message = $this->Thread->findById($message_id);

				$last_thread = $this->ThreadsMessage->find('first', array(
						'joins' => array(
								array(
										'type' => 'inner',
										'table' => '(select MAX(created_time) as maxdate, thread_id from threads_messages where facebook_user_id != '.$message['Thread']['facebook_user_recipient_id'].' group by thread_id)',
										'alias' => 'ThreadsMessageMaxDate',
										'conditions' => array(
												'ThreadsMessageMaxDate.thread_id = ThreadsMessage.thread_id and ThreadsMessageMaxDate.maxdate = created_time',
												'ThreadsMessage.thread_id' =>$message['Thread']['id']
										)
								)
						),
						'contain' => array('FacebookUser')
				));

				$last_thread['ThreadsMessage']['mood'] = $message_mood;
				$last_thread['ThreadsMessage']['category'] = $message_category;
				$this->ThreadsMessage->save($last_thread);
					
				//Fix for Webservice
				$text = str_replace('"', '', $last_thread['ThreadsMessage']['message']);
					
				$url .= "text=".urlencode($text)."&polarity=".$message_mood."&Category=".$message_category;
				$result = $this->Utilities->webservicePost($url);
					
				$result = json_decode($result,true);
					
				if($result['status'] != Constants::$WS_OK){
					CakeLog::error('submitTraining');
				}
					
				return $this->reloadThread($message['Thread']['id']);
			}
				
		}
	}



	
	public function send_comment() {
		$this->autoRender = false;
	  
		if(!empty($this->data)){
			$comment = array();
			$comment['event_id'] = $this->data['event_id'];
			$comment['user_id'] = $this->data['user_id'];
			$comment['text'] = $this->data['message_text'];
			if ($this->Event->exists($comment['event_id']) && $this->User->exists($comment['user_id'] ) ){
				if($this->EventsComment->save($comment)){
					
					$this->Session->setFlash(__("Comentari publicat correctament"),'flash_success');
				}else{
					$this->Session->setFlash(__("Error al publicar el comentari, torna-ho a intentar"),'flash_warning');
				}
				
			}else{
				$this->Session->setFlash(__("O l'usuari o l'event no existeixen"),'flash_fail');
				
			}
			
			$this->redirect(array('action' => 'viewEvent',$comment['event_id'] ));
			 
				
				
		}
	}

	public function changeStatus(){
		$this->autoRender = false;
		if( $this->request->is('ajax') ) {
			$id = $this->request->data('id');
			$event = $this->request->data('event');
			if ($id != null && $event != null){
				$current_user = $this->Auth->user();
				$user_event = $this->UserEvent->find('first', array(
						'conditions' => array('user_id' => $current_user['id'], 'event_id' => $event)));
				if ($user_event['UserEvent']['id'] != null){
					$this->UserEvent->id = $user_event['UserEvent']['id'];
					if($this->UserEvent->saveField('status_id', $id)){
						return ok;
					}
				}else{
					$user_event['user_id'] = $current_user['id'];
					$user_event['status_id'] = $id;
					$user_event['event_id'] = $event;
					if($this->UserEvent->save($user_event)){
						return ok;
					}
				}
			}
		}
		
	}
	
	public function eventRead(){
		$this->autoRender = false;
		if( $this->request->is('ajax') ) {
			$current_user = $this->Auth->user();
			$event = $this->request->data('event');
			if ($event != null){
				$user_event = $this->UserEvent->find('first', array(
						'conditions' => array('user_id' => $current_user['id'], 'event_id' => $event)));
				if ($user_event['UserEvent']['id'] == null){
					$this->UserEvent->create();
					$user_event_save['user_id'] = $current_user['id'];
					$user_event_save['status_id'] = Constants::$USER_EVENT_READ;
					$user_event_save['event_id'] = $event;
					if($this->UserEvent->save($user_event_save)){
						return ok;
					}else{
						return false;
					}
				}
			}
		}
		
	}
	public function noMore ($id = null){
		$this->autoRender = false;
		$user = $this->Auth->user();
		if (isset($this->request->data['id'])){
			$id = $this->request->data['id'];
			if ($this->Event->exists($id)){
				$rs_id = $this->UserEvent->find('first', array(
						'conditions' => array(
								'user_id' => $user['id'], 'status_id' =>Constants::$USER_EVENT_RECOMMENDED, 'event_id' => $id)));
				$this->UserEvent->id = $rs_id['UserEvent']['id'];
				if ($this->UserEvent->saveField('sr_score', '-1.0')){
					return true;
				}else return false;
			}else{
				return false;
			}
		}else{
			return false;
		}
	}
	
	public function noMoreSimilitudes (){
		//aqui aniria el codi x fer q els similars anéssin fora
	}
	

}
