<?php
App::uses('AppController', 'Controller');
App::uses('CakeEmail', 'Network/Email');

/**
 * Users Controller
 *
 * @property User $User
 * @property PaginatorComponent $Paginator
 */
class UsersController extends AppController {

/**
 * Components
 *
 * @var array
 */
	public $components = array('Paginator');
	var $uses = array('Role','User', 'UsersNotification', 'UserEvent', 'Event', 'UserPreference', 'City', 'UserUser');
	var $valid_formats = array("jpg","png");
	

	
	function beforeFilter() {
		parent::beforeFilter();
		$this->Auth->allow('register');
		$this->__securityConf();
		
	}
	
	/**************** SECURITY FUNCTIONS ****************/
	
	public function isAuthorized($user) {
		
		$action = $this->request->action;
		if ($action == "register") {
			 return parent::isAuthorized($user);
		}
		
		if($action == "deleteUser" && $this->request->is("post") &&
			($user['role_id'] == Constants::$ROLE_USER )) {
			
			$this->AjaxException->throwException(401,'ERROR','Unauthorized');
		}	
		    	
	    return parent::isAuthorized($user);
	}
	
	private function __securityConf(){
		
		$this->Security->unlockedActions = array('deleteUser', 'addPreferences', 'viewMine','checkExpirationTokens','markNotificationsReaded', 'configureNotifies', 'register', 'viewUser','addFriend', 'listFriends', 'findFriendByName', 'listUsers');
		
	}
	
	/************** END SECURITY FUNCTIONS **************/
	
	
	/**
	 * index method
	 *
	 * @return void
	 */
	public function index() {
		$this->set('users', $this->Paginator->paginate());
		$this->layout = 'default';
		$this->set('current_user', $this->Auth->user());
		$can_remove = true;
		if($user['role_id'] == Constants::$ROLE_USER ) {
			$can_remove = false;
		}
		
		$this->set('can_remove', $can_remove);
		
	}
	
	/*private function _facebookConnect(){
		Facebook\FacebookSession::setDefaultApplication(Constants::$APP_ID,Constants::$SECRET_ID);
	}*/
	
	
	/**
	 * login method
	 *
	 * @return void
	 */
	
	function login() {
		$this->set('title_for_layout', __('Entra'));
		$this->layout = 'login';
		
		if($this->Auth->loggedIn()){
			$this->redirect(array('controller'=>'pages','action'=>'index'));
		}
		
		if ($this->request->is('post')) {
			if ($this->Auth->login()) {
				$this->Session->write('Auth.User.timezone', $this->request->data['User']['timezone']);
				$current_user = $this->Auth->user();
					
				$this->Auth->login($current_user); // Creem una variable amb el usuari actual
				
				$this->redirect(array('controller'=>'events','action'=>'mainPage'));
				//$this->checkExpirationTokens();
		
			} else {
				$this->Auth->logout();
				$this->Session->setFlash(__('Usuari o contrasenya incorrecta, si us plau, torna-ho a provar'),'flash_fail');
			}
		}
	}
	
	public function register(){
	
		if ($this->request->is('post')) {
			if (isset($this->request->data['t_and_c']) && $this->request->data['t_and_c'] == Constants::$ACCEPT_TERMS){
				if(isset($this->request->data['User'])){
					if (empty($this->User->findByUsername($this->request->data['User']['username']))){
						$user = array();
						$user['username'] = $this->request->data['User']['username'];
						$user['name'] = $this->request->data['User']['name'];
						$user['surname'] = $this->request->data['User']['surname'];
						$user['password'] = $this->request->data['User']['password'];
						$user['role_id'] = Constants::$ROLE_USER;
						$user['city_id'] = Constants::$USER_CITY;
						$user['active'] = 1;
						$this->User->create();
						if($this->User->save($user)){
							$this->Session->setFlash(__('Usuari registrat correctament'),'flash_success');
							$this->redirect(array('controller'=>'users','action'=>'login'));
						}else{
							throw new BadRequestException('Usuari mal construit');
						}
					}else{
						throw new BadRequestException('Aquest email ja existeix');
						
					}
				
				}else{
					throw new BadRequestException('Registre mal construit');
				}
			}else{
				$this->redirect(array('controller'=>'users','action'=>'index'));
			}
			
		}else{
			$this->redirect(array('controller'=>'users','action'=>'index'));
		}
		
	}
	
	/**
	 * logout method
	 *
	 * @return void
	 */
	function logout() {
		CakeSession::destroy();
		$this->redirect($this->Auth->logout());
	}
	
	
	
	/**
	 * forget password method
	 *
	 * @return void
	 */
	public function forgetpwd(){
		$this->layout = 'login';

		if(!empty($this->data))
		{
			if((!isset($this->data['User']['username'])) || $this->data['User']['username'] == '')
			{
				$this->Session->setFlash(__('Introdueixi el correu electrònic amb el qual et vas registrar'),'flash_fail');
			}
			else
			{
				$delegate = false;
				$username = $this->data['User']['username'];
	
				$user_pwd = $this->User->find('first',array('conditions'=>array('User.username'=>$username)));
				if(isset($user_pwd) && (sizeof($user_pwd) > 0)){
					if($user_pwd['User']['role_id'] == Constants::$ROLE_DELEGATE) $delegate = true;
				}
				
				if($chief)
				{
					if($user_pwd['User']['active'])
					{
						$key = Security::hash(String::uuid(),'sha512',true);
						$user_pwd['User']['tokenhash'] = $key;
						$this->User->id = $user_pwd['User']['id'];
						if($this->User->saveField('tokenhash',$user_pwd['User']['tokenhash'])){
								
							$this->Session->setFlash(__('Li hem enviat un correu electrònic amb un enllaç per establir una nova constrassenya'),'flash_success');
							$this->redirect(array('controller'=>'users','action'=>'login'));
						}
						else{
							$this->Session->setFlash(__('S&#39ha produit un error'),'flash_fail');
						}
					}
					else
					{
						$this->Session->setFlash(__('El compte no està actiu'),'flash_fail');
					}
				}
				else
				{
					$this->Session->setFlash(__('No existeix cap compte amb aquest correu electrònic associat'),'flash_fail');
				}
			}
		}
	}
	

	
/**
 * view method
 *
 * @throws NotFoundException
 * @param string $id
 * @return void */
 
	public function viewUser($id = null) {
		$current_user = $this->Auth->user();
		$this->set('page','view_user');
		if (!$this->User->exists($id)) {
			throw new NotFoundException(__('Usuari no vàlid'));
		}else{
			if ($id != $current_user['id']){
				$options = array('conditions' => array('User.' . $this->User->primaryKey => $id, 'active' => 1));
				$user = $this->User->find('first', $options);
				$this->set('title_for_layout', $user['User']['name']);
				$this->set('user', $user);
				
				$user_preferences = $this->UserPreference->findByUserId($id);
				if (!empty($user_preferences)){
					$preferences = rtrim($user_preferences['UserPreference']['categories'], ",}");
					$preferences = ltrim ($preferences, '{');
					$preferences =  explode(',', $preferences);
					$preferences = $this->analizePreferences($preferences);
					$this->set ('preferences',$preferences );
				}
				
				$user_events = $this->UserEvent->getEventsFromUser($user['User']['id']);
				
				foreach ($user_events as $key=>$user_event){
					if ($user_event['Event']['id'] != null){
						$user_events[$key]['City'] = $this->City->findById($user_event['Event']['city_id'])['City'];
					}
				}
				$this->set('user_events', $user_events);
				if (isset($user['User']['city_id'])){
					$city = $this->City->findById($user['User']['city_id']);
				}else{
					$city = __("Ciutat no especificada");
				}
				$friend = $this->UserUser->find('first', array(
						'conditions' => array(
								'user_id' => $current_user['id'],
								'friend_user_id' => $id,
								'active' => 1
								)));
				if (!empty($friend)){
					$this->set('are_friends', true);
				}else{
					$this->set('are_friends', false);
				}
				$this->set('city',$city);
				$roles = $this->User->Role->find('list');
				$this->set('roles', $roles);
			}else{
				$this->redirect(array('action' => 'viewMine'));
			}
		}
		

	}

	private function addPreferences ($user_data){
		$preferences = "{";
		if (isset($user_data['concerts_checkbox']) && $user_data['concerts_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_CONCERTS.',';
		}
		if (isset($user_data['theatre_checkbox']) && $user_data['theatre_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_THEATRE.',';
		}
		if (isset($user_data['conferences_checkbox']) && $user_data['conferences_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_CONFERENCES.',';
		}
		if (isset($user_data['classes_checkbox']) && $user_data['classes_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_CLASSES.',';
		}
		if (isset($user_data['infantil_checkbox']) && $user_data['infantil_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_INFANTIL.',';
		}
		if (isset($user_data['fires_checkbox']) && $user_data['fires_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_FIRES.',';
		}
		if (isset($user_data['sports_checkbox']) && $user_data['sports_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_SPORTS.',';
		}
		if (isset($user_data['gastro_checkbox']) && $user_data['gastro_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_GASTRO.',';
		}
		if (isset($user_data['literature_checkbox']) && $user_data['literature_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_LITERATURE.',';
		}
		if (isset($user_data['cinema_checkbox']) && $user_data['cinema_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_CINEMA.',';
		}
		if (isset($user_data['museums_checkbox']) && $user_data['museums_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_MUSEUMS.',';
		}
		if (isset($user_data['ong_checkbox']) && $user_data['ong_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_ONG.',';
		}
		if (isset($user_data['festamajor_checkbox']) && $user_data['festamajor_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_FESTAMAJOR.',';
		}
		if (isset($user_data['others_checkbox']) && $user_data['others_checkbox'] == Constants::$ACTIVE ){
			$preferences .= Constants::$CATEGORY_OTHER.',';
		}
		$preferences .= "}";
		return $preferences;
	}
	
	private function analizePreferences($array_preferences){
		
		$preferences = array();
		$preferences['concert'] = false;
		$preferences['theatre'] = false;
		$preferences['conferences'] = false;
		$preferences['classes']  = false;
		$preferences['infantil'] = false;
		$preferences['fires'] = false;
		$preferences['gastro'] = false;
		$preferences['gastro'] = false;
		$preferences['sports'] = false;
		$preferences['literature'] = false;
		$preferences['cinema'] = false;
		$preferences['museums'] = false;
		$preferences['ong'] = false;
		$preferences['festamajor'] = false;
		$preferences['other'] = false;
		
		foreach($array_preferences as $array_preference){
			if ($array_preference == Constants::$CATEGORY_CONCERTS){
				$preferences['concert'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_THEATRE){
				$preferences['theatre'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_CONFERENCES){
				$preferences['conferences'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_CLASSES){
				$preferences['classes'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_INFANTIL){
				$preferences['infantil'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_FIRES){
				$preferences['fires'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_GASTRO){
				$preferences['gastro'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_GASTRO){
				$preferences['gastro'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_SPORTS){
				$preferences['sports'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_LITERATURE){
				$preferences['literature'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_CINEMA){
				$preferences['cinema'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_MUSEUMS){
				$preferences['museums'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_ONG){
				$preferences['ong'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_FESTAMAJOR){
				$preferences['festamajor'] = true;
			}
			elseif ($array_preference == Constants::$CATEGORY_OTHER){
				$preferences['other'] = true;
			}
		}
		return $preferences;
	}
	
	/**
	 * viewMine method
	 *
	 * @return void
	 */
	
	public function viewMine() {
		$this->set('title_for_layout', __('El meu perfil'));
		
		
		$this->set('page','view_mine');
		$current_user = $this->Auth->user();
		$this->User->create();
		if ($this->request->is('post')) {
			$user = array();
			//Per no perdre la foto anterior si en l'actual no n'hi ha
			if($this->request->data['User']['photo'] == ''){
				unset($this->request->data['User']['photo']);
			    unset($this->request->data['User']['photo_dir']);
			}else{
				$user['photo'] =  $this->request->data['User']['photo'];
				$user['photo_dir'] =  $this->request->data['User']['photo_dir'];
			}	
	

			if (isset($this->request->data['User']['id'])) {

				$user['id'] = $this->request->data['User']['id'];
				$user['name'] =  $this->request->data['User']['name'];
				$user['username'] =  $this->request->data['User']['username'];
				$user['password'] =  $this->request->data['User']['password'];
				$user['surname'] =  $this->request->data['User']['surname'];
				$user['description'] =  $this->request->data['User']['description'];
				$user['profesion'] =  $this->request->data['User']['profesion'];
				$user['city'] =  Constants::$USER_CITY;
				
				$this->User->save($user);
				$preferences = $this->UserPreference->findByUserId($user['id']);
				if (empty($preferences)){
					$this->UserPreference->create();
					$preferences['UserPreference']['user_id'] = $user['id'];
				}else{
					$this->UserPreference->id = $preferences['UserPreference']['id'];
				}
				$preferences['UserPreference']['categories'] = $this->addPreferences ($this->request->data['User']);
				$this->UserPreference->save($preferences);
				
			    if($current_user['id'] == $this->request->data['User']['id']){	
		    	
			      	$current_user = $this->User->read(null, $this->Auth->User('id'));
			        $current_user = $current_user['User'];
					$this->Auth->login($current_user);
				}
						    
				$this->Session->setFlash(__('Usuari guardat/modificat correctament'),'flash_success');
				$this->redirect(array('controller'=>'users','action' => 'viewMine'));
			}		    
						
		}
		$user_events = 	$user_events = $this->UserEvent->getEventsFromUser($current_user['id']);
		foreach ($user_events as $key=>$user_event){
			if ($user_event['Event']['id'] != null){
				$user_events[$key]['City'] = $this->City->findById($user_event['Event']['city_id'])['City'];
			}
		}	
		$this->set('user_events', $user_events);
		$this->set('errors', $this->User->validationErrors);
		if(isset($this->request->query['id']) && isset($this->request->query['role'])){
			$this->set('id', $this->request->query['id']);
			$this->set('role', $this->request->query['role']);
		}	
		$roles = $this->User->Role->find('list');
		$this->set(compact('roles'));
		
		$can_remove = true;
		if($current_user['role_id'] == Constants::$ROLE_USER ) {
			$can_remove = false;
		}
		$current_user = $this->Auth->user();
		if (isset($current_user['city_id'])){
			$city = $this->City->findById($current_user['city_id']);
		}else{
			$city = __("Ciutat no especificada");
		}
		$user_preferences = $this->UserPreference->findByUserId($current_user['id']);
		if (isset($user_preferences['UserPreference'])){
			$preferences = rtrim($user_preferences['UserPreference']['categories'], ",}");
			$preferences = ltrim ($preferences, '{');
			$preferences =  explode(',', $preferences);
			$preferences = $this->analizePreferences($preferences);
			$this->set('preferences', $preferences);
		}
		$this->set('current_user', $current_user);
		$this->set('can_remove', $can_remove);
		$this->set('city', $city);
		

}
	
	public function addFriend ($id = null){
		$this->autoRender = false;
		if (!isset($id)){
			$id = $this->request->data['id'];
		}
		if (!$this->User->exists($id)) {
			throw new NotFoundException(__('Usuari no vàlid'));
		}else{
			$this->UserUser->create();
			$friend['friend_user_id'] = $id;
			$current_user = $this->Auth->user();
			$friend['user_id'] = $current_user['id'];
			if($this->UserUser->save($friend)){
				$this->UserUser->create();
				$friend['friend_user_id'] = $current_user['id'];
				$friend['user_id'] = $id;
				if($this->UserUser->save($friend)){
					$this->Session->setFlash(__('Felicitats! Has afegit una nova amistat'),'flash_success');
				}else{
					return false;
				}
			}else{
				$this->Session->setFlash(__("No s'ha pogut efectuar l'amistat, torna-ho a intentar"),'flash_fail');
			}
			
		}
			
		
	}
/**
 * edit method
 *
 * @throws NotFoundException
 * @param string $id
 * @return void
 */
	public function edit($id = null, $idDelegate = null) {
		if (!$this->User->exists($id)) {
			$this->Session->setFlash(__('L&#39usuari seleccionat no existeix'),'flash_fail');
		}
		
		if ($this->request->is(array('post'))) {
			
			$this->request->data['User']['id'] = $id;
			$this->request->data['Delegate']['id'] = $idDelegate;
			
			if(isset($this->request->data['Delegate']['delegate_id']) && $this->request->data['Delegate']['delegate_id'] == ''){
				$actualDelegate = $this->Delegate->findByUserId($this->Auth->user('id'));
				$this->request->data['Delegate']['delegate_id'] = $actualDelegate['Delegate']['id'];
			}
			
			if($this->request->data['Delegate']['id'] === $this->request->data['Delegate']['delegate_id']){
				$this->Session->setFlash(__('Error a l&#39actualitzar l&#39usuari: el delegat i el seu pare són iguals'),'flash_fail');
			}else{
			
				if ($this->User->save($this->request->data) && $this->Delegate->save($this->request->data)) {
					$this->Session->setFlash(__('L&#39usuari s&#39ha guardat correctament'),'flash_success');
					$this->redirect(array('controller'=>'users','action' => 'userslists'));
				} else {
					$this->Session->setFlash(__('S&#39ha produit un error a l&#39hora de guardar l&#39usuari'));
				}
			}
		} 
		
		$this->redirect(array('controller'=>'users','action' => 'userslists'));
	}

/**
 * delete method
 *
 * @throws NotFoundException
 * @param string $id
 * @return void
 */
	public function delete($id = null) {
		
		if (!isset($id)){
			$id = $this->request->data['id'];
		}
		$this->User->id = $id;
		if (!$this->User->exists()) {
			$this->Session->setFlash(__('L&#39usuari seleccionat no existeix'),'flash_fail');
		}
		
		$this->request->onlyAllow('post', 'delete');
		
		if($this->User->saveField('active',0)){
			$this->Session->setFlash(__('Usuari eliminat correctament'),'flash_success');
		} else {
			$this->Session->setFlash(__('No s&#39ha pogut eliminar l&#39usuari'),'flash_fail');
		}
		$this->redirect(array('controller'=>'users','action' => 'add'));
		//return 1;
	}
	
	
	
	private function getExt($name,$path=null){
		try{
			if($path!=null){
				$image = $path;
			}else{
				$image = $name;
			}
	
			if(file_exists($image)){
				$sizeFileImage = getimagesize($image);
				if($sizeFileImage){
					if($sizeFileImage["mime"] == "image/jpeg"){
						$nameaux = 'jpg';
					}
					if($sizeFileImage["mime"] == "image/png"){
						$nameaux = 'png';
					}
				}else{
					$continue = false;
				}
			}else{
				$nameaux=str_replace(' ','_',strtolower($name));
				$substrings = explode(".", $nameaux);
				if(count($substrings)>0){
					$nameaux=$substrings[count($substrings)-1];
				}
			}
		} catch (Exception $e) {
			$nameaux=str_replace(' ','_',strtolower($name));
			$substrings = explode(".", $nameaux);
			if(count($substrings)>0){
				$nameaux=$substrings[count($substrings)-1];
			}
			if(strlen($ext)>0 && !in_array($ext, $this->valid_formats)){
				$nameaux = '';
			}
		}
		$nameaux = $nameaux.'';
		return $nameaux;
	}
	
	
	private function getImage($imageName,$path=null){
		try{
			$ext = $this->getExt($imageName,$path);
	
			if(strlen($ext)>0 && in_array($ext, $this->valid_formats)){
				if(isset($imageName) && strlen(trim($imageName))>0){
					if($path!=null){
						$image = $path;
					}else{
						$image = $imageName;
					}
				}
				if( $ext=="jpg"){
					$srcimage = imagecreatefromjpeg($image);
				}else{
					if($ext=="png"){
						$srcimage = imagecreatefrompng($image);
					}else{
						$srcimage = imagecreatefrompng(WWW_ROOT."/img/no_image.png");
					}
				}
			}else{
				$srcimage = imagecreatefrompng(WWW_ROOT."/img/no_image.png");
			}
		} catch (Exception $e) {
			$srcimage = imagecreatefrompng(WWW_ROOT."/img/no_image.png");
		}
		return $srcimage;
	
	}

	
	public function listFriends(){
		$this->set('page','list_friends');
		 $this->set('title', 'Amistats');
		$current_user = $this->Auth->user();
		$friends_count = 0;
		$friends = null;
		 
		if($this->request->is('ajax')){
			$this->autoRender = false;
			$friends = $this->UserUser->find('all', array(
					'conditions' => array(
							'UserUser.user_id' => $current_user['id'],
							'UserUser.active' => 1
							),
					'joins' => array(
        				array(
				            'table' => 'users',
				            'alias' => 'User',
				            'type' => 'INNER',
				            'conditions' => array(
				                'User.id = UserUser.friend_user_id'
				            )
       					 )
    				),
					'fields' => array('User.id', 'User.photo', 'User.name', 'User.surname', 'User.photo_dir'
							)
					));
			$friends_count = $this->UserUser->find('count', array(
					'conditions' => array(
							'UserUser.user_id' => $current_user['id'],
							'UserUser.active' => 1
							)));
			$this->set('friends', $friends);
			$this->set('friends_count', $friends_count);
			$this->render('/Elements/list_profiles',false);
		}
		else{
			$this->set('friends', $friends);
			$this->set('friends_count', $friends_count);
		}
	}
	
	public function listUsers(){
		$this->set('page','list_users');
		$this->set('title', 'Usuaris');
		$users_count = 0;
		$users = null;
		$current_user = $this->Auth->user();
		if($this->request->is('ajax')){
			$this->autoRender = false;
			$users = $this->User->find('all', array(
					'conditions' => array(
							'User.id !=' => $current_user['id'],
							'User.active' => 1
					)
			));
			$users_count = $this->User->find('count', array(
					'conditions' => array(
							'User.id !=' => $current_user['id'],
							'User.active' => 1
					)
			));
			$this->set('users', $users);
			$this->set('users_count', $users_count);
			$this->render('/Elements/list_profiles',false);
		}
		else{
			$this->set('users', $users);
			$this->set('users_count', $users_count);
		}
	}
	
	
	public function deleteUser($id = null) {
		$this->autoRender = false;
		
		if (!isset($id)){
			$id = $this->request->data['id'];
		}
		$this->User->id = $id;
		if (!$this->User->exists()) {
			$this->Session->setFlash(__('L&#39usuari seleccionat no existeix'),'flash_fail');
		}
		
		$this->request->onlyAllow('post', 'delete');
		
		if($this->User->saveField('active',0)){
			$this->Session->setFlash(__('Usuari eliminat correctament'),'flash_success');
		} else {
			$this->Session->setFlash(__('No s&#39ha pogut eliminar l&#39usuari'),'flash_fail');
		}
		//$this->redirect(array('controller'=>'users','action' => 'add'));
		
		
		
	}
	
	public function userDetail($id = null) {
		$id = $this->request->data['id'];
		
		if($id != 0){
			$user = $this->User->find('first',array('conditions' => array('id' => $id)));
			$this->set('user', $user);
			$role = $this->Role->find('first',array('conditions' => array('id' => $this->request->data['role'])));
			$this->set('user_role', $role);
		}

		$roles = $this->Role->find('list',array('fields' => array('id','name')));
		$this->set('roles', $roles);
		
		$this->render('/Elements/userDetail',false);
	}
	
	
	public function findFriendByName(){
		
		$current_user = $this->Auth->user();
		if($this->request->is('get')){
			 
			$text = $this->request->query['text'];
			 
			$friends = $this->User->find('all', 
					array('conditions' => array(
							'OR' => array(
									'User.name LIKE' => '%'.$text.'%',
									 'User.surname LIKE' => '%'.$text.'%',
									'User.fullname LIKE' => '%'.$text.'%',
									),
							 'User.active' => 1),
							'joins' => array(
									array(
											'table' => 'user_users',
											'alias' => 'UserUser',
											'type' => 'INNER',
											'conditions' => array(
													'User.id = UserUser.friend_user_id', 
													'UserUser.user_id' => $current_user['id']
										
											)
									)
							),
							));
			if (!empty($friends)){
				$friend_first = $friends[0];
				foreach($friends as $key=> $friend){
					if ($friend_first['User']['id'] == $friend['User']['id']){
						if ($friend !== reset($friends)){
							unset ($friends[$key]);
						}
					}else{
						$friend_first = $friend;
					}
				}
				$friends_count = count ($friends);
			}else{
				$friends_count = 0;
			}
			$this->set('friends', $friends);
			$this->set('friends_count', $friends_count);
			$this->render('/Elements/list_profiles',false);
		}
	}
	
	public function findUserByName(){
	
		$current_user = $this->Auth->user();
		if($this->request->is('get')){
	
			$text = $this->request->query['text'];
	
			$users = $this->User->find('all',
					array('conditions' => array(
							'OR' => array(
									'User.name LIKE' => '%'.$text.'%',
									'User.surname LIKE' => '%'.$text.'%',
									'User.fullname LIKE' => '%'.$text.'%',
							),
							'User.active' => 1
					)));
			$users_count = $this->User->find('count',
					array('conditions' => array(
							'OR' => array(
									'User.name LIKE' => '%'.$text.'%',
									'User.surname LIKE' => '%'.$text.'%',
									'User.fullname LIKE' => '%'.$text.'%',
							),
							'User.active' => 1
					)));
			$this->set('users', $users);
			$this->set('users_count', $users_count);
			$this->render('/Elements/list_profiles',false);
		}
	}
		
		
		
	public function markNotificationsReaded(){
		$this->autoRender = false;
		
		$user_id = $this->data['user_id'];
		
		$unreaded_notifications = $this->User->Notification->updateAll(
				array('read' => 1),
				array(
						'user_id' => $user_id,
						'read' => 0
				)
		);
		
		if($unreaded_notifications){
			return 'ok';
		}
		
		$this->AjaxException->throwException(500,'ERROR','Error');
		
	}

	public function configureNotifies($id = null) {

		if ($id == null){
            $current_user = $this->Auth->user();
            $id = $current_user['id'];
		}

		if($id != 0){
			$user = $this->UsersNotification->find('first',array('conditions' => array('user_id' => $id)));
			$this->set('user_notification', $user);
		}

        $this->set('page', 'edit_notification');
	}
	


}	
