<?php
App::uses('AppModel', 'Model');
/**
 * User Model
 *
 * @property Role $Role
 */
class User extends AppModel {

    public $actsAs = array(
            'Upload.Upload' => array(
                    'photo' => array(
                            'fields' => array(
                                    'dir' => 'photo_dir'
                            ),
                            'thumbnailSizes' => array(
                                    'small' =>'35x35',
                                    'normal' =>'100x100',
                            ),
                            'thumbnailMethod'=> 'php',
                            'deleteOnUpdate' => 'true'
                    )
            ),
    		'Notification.Notifiable' => array(
    				'subjects' => array('User')
    		),
    );
    
    public $virtualFields = array(
    		'fullname' => 'CONCAT(User.name, " ", User.surname)'
    );
    
/**
 * Validation rules
 *
 * @var array
 */
	public $validate = array(
		'role_id' => array(
			'numeric' => array(
				'rule' => array('numeric'),
				//'message' => 'Your custom message here',
				//'allowEmpty' => false,
				//'required' => false,
				//'last' => false, // Stop validation after this rule
				//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'active' => array(
			'boolean' => array(
				'rule' => array('boolean'),
				//'message' => 'Your custom message here',
				//'allowEmpty' => false,
				//'required' => false,
				//'last' => false, // Stop validation after this rule
				//'on' => 'create', // Limit validation to 'create' or 'update' operations
			),
		),
		'username' => array(
				'required' => array(
						'rule' => array('notEmpty'),
						'message' => '* Aquest camp és obligatori'
				),
				'email' => array(
						'rule' => array('email'),
						'message' => '* Entra una direcció e-mail vàlida'
				),
				'unique' => array(
				        'rule'    => 'isUnique',
				        'message' => 'Aquest nom d&#39usuari ja està agafat.',
				        'on' => 'create'
				)
		),
		'password' => array(
        		'minLength' => array(
        		        'rule' => array('minLength',5),
        		        'message' => '* Ha de tenir com a mínim 5 caràcters'
        		),
				'required' => array(
						'rule' => array('notEmpty'),
						'message' => '* Aquest camp és obligatori'
				),
				
		),
		'name' => array(
		        'required' => array(
		                'rule' => array('notEmpty'),
		                'message' => '* Aquest camp és obligatori'
		        )
		),
		'surname' => array(
		        'required' => array(
		                'rule' => array('notEmpty'),
		                'message' => '* Aquest camp és obligatori'
		        )
		)/*,
		'photo' => array(
		        'required' => array(
		                'rule' => array('notEmpty'),
		                'message' => '* Aquest camp és obligatori'
		        )
		)*/
	);
	

	//The Associations below have been created with all possible keys, those that are not needed can be removed

/**
 * belongsTo associations
 *
 * @var array
 */
	public $belongsTo = array(
		'Role' => array(
			'className' => 'Role',
			'foreignKey' => 'role_id',
			'conditions' => '',
			'fields' => '',
			'order' => ''
		),
		'City' => array(
				'className' => 'City',
				'foreignKey' => 'city_id',
				'conditions' => '',
				'fields' => '',
				'order' => ''
		)
	);
	
	
	public $hasMany = array(
		'Chat' => array(
			'className' => 'Chat',
			'foreignKey' => 'user_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => 'created_time',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
		),
		'UserEvent' => array(
			'className' => 'UserEvent',
			'foreignKey' => 'user_id',
			'dependent' => false,
			'conditions' => '',
			'fields' => '',
			'order' => 'created_time',
			'limit' => '',
			'offset' => '',
			'exclusive' => '',
			'finderQuery' => '',
			'counterQuery' => ''
			),
			'EventsComment' => array(
					'className' => 'EventsComment',
					'foreignKey' => 'user_id',
					'dependent' => false,
					'conditions' => '',
					'fields' => '',
					'order' => 'created_time',
					'limit' => '',
					'offset' => '',
					'exclusive' => '',
					'finderQuery' => '',
					'counterQuery' => ''
			),
			
	);

/**
 * hasAndBelongsToMany associations
 *
 * @var array
 */


	public function beforeSave($options = array()) {
		if (isset($this->data['User']['password'])) {
			$this->data['User']['password'] = AuthComponent::password($this->data['User']['password']);
		}
		return true;
	}
	
	public function GetData() {
		/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
		 * Easy set variables
		*/
	
		/* Array of database columns which should be read and sent back to DataTables. Use a space where
		 * you want to insert a non-database field (for example a counter or static image)
		*/
		$aColumns = array('username', 'role_id', 'created', 'active');
	
		/* Indexed column (used for fast and accurate table cardinality) */
		$sIndexColumn = "id";
	
		/* DB table to use */
		$sTable = "users";
	
		App::uses('ConnectionManager', 'Model');
		$dataSource = ConnectionManager::getDataSource('default');
	
		/* Database connection information */
		$gaSql['user'] = $dataSource->config['login'];
		$gaSql['password'] = $dataSource->config['password'];
		$gaSql['db'] = $dataSource->config['database'];
		$gaSql['server'] = $dataSource->config['host'];
		$gaSql['encoding'] = $dataSource->config['encoding'];
	
		/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
		 * If you just want to use the basic configuration for DataTables with PHP server-side, there is
		* no need to edit below this line
		*/
	
		/*
		 * Local functions
		*/
	
		function fatal_error($sErrorMessage = '') {
			header($_SERVER['SERVER_PROTOCOL'] . ' 500 Internal Server Error');
			die($sErrorMessage);
		}
	
		/*
		 * MySQL connection
		*/
		if (!$gaSql['link'] = mysql_pconnect($gaSql['server'], $gaSql['user'], $gaSql['password'])) {
			fatal_error('Could not open connection to server');
		}
	
		if (!mysql_select_db($gaSql['db'], $gaSql['link'])) {
			fatal_error('Could not select database ');
		}
	
		/*
		 * MySQL encoding to UTF8
		*/
		if (!mysql_set_charset("utf8", $gaSql['link'])) {
			fatal_error('Could not encoding database connection');
		}
	
		/*
		 * Paging
		*/
		$sLimit = "";
		if (isset($_GET['iDisplayStart']) && $_GET['iDisplayLength'] != '-1') {
			$sLimit = "LIMIT " . intval($_GET['iDisplayStart']) . ", " . intval($_GET['iDisplayLength']);
		}
	
		/*
		 * Ordering
		*/
		$sOrder = "";
		if (isset($_GET['iSortCol_0'])) {
			$sOrder = "ORDER BY  ";
			for ($i = 0; $i < intval($_GET['iSortingCols']); $i++) {
				if ($_GET['bSortable_' . intval($_GET['iSortCol_' . $i])] == "true") {
					$sOrder .= "`" . $aColumns[intval($_GET['iSortCol_' . $i])] . "` " . ($_GET['sSortDir_' . $i] === 'asc' ? 'asc' : 'desc') . ", ";
				}
			}
	
			$sOrder = substr_replace($sOrder, "", -2);
			if ($sOrder == "ORDER BY") {
				$sOrder = "";
			}
		}
	
		/*
		 * Filtering
		* NOTE this does not match the built-in DataTables filtering which does it
		* word by word on any field. It's possible to do here, but concerned about efficiency
		* on very large tables, and MySQL's regex functionality is very limited
		*/
		$sWhere = "";
		if (isset($_GET['sSearch']) && $_GET['sSearch'] != "") {
			$sWhere = "WHERE (";
			for ($i = 0; $i < count($aColumns); $i++) {
				$sWhere .= "`" . $aColumns[$i] . "` LIKE '%" . mysql_real_escape_string($_GET['sSearch']) . "%' OR ";
			}
			$sWhere = substr_replace($sWhere, "", -3);
			$sWhere .= ')';
		}
	
		/* Individual column filtering */
		for ($i = 0; $i < count($aColumns); $i++) {
			if (isset($_GET['bSearchable_' . $i]) && $_GET['bSearchable_' . $i] == "true" && $_GET['sSearch_' . $i] != '') {
				if ($sWhere == "") {
					$sWhere = "WHERE ";
				}
				else {
					$sWhere .= " AND ";
				}
				$sWhere .= "`" . $aColumns[$i] . "` LIKE '%" . mysql_real_escape_string($_GET['sSearch_' . $i]) . "%' ";
			}
		}
	
		/*
		 * SQL queries
		* Get data to display
		*/
		$sQuery = "
		SELECT SQL_CALC_FOUND_ROWS `" . str_replace(" , ", " ", implode("`, `", $aColumns)) . "`
		FROM   $sTable
		$sWhere
		$sOrder
		$sLimit
		";
		$rResult = mysql_query($sQuery, $gaSql['link']) or fatal_error('MySQL Error: ' . mysql_errno());
	
		/* Data set length after filtering */
		$sQuery = "
		SELECT FOUND_ROWS()
		";
		$rResultFilterTotal = mysql_query($sQuery, $gaSql['link']) or fatal_error('MySQL Error: ' . mysql_errno());
		$aResultFilterTotal = mysql_fetch_array($rResultFilterTotal);
		$iFilteredTotal = $aResultFilterTotal[0];
	
		/* Total data set length */
		$sQuery = "
		SELECT COUNT(`" . $sIndexColumn . "`)
		FROM   $sTable
		";
		$rResultTotal = mysql_query($sQuery, $gaSql['link']) or fatal_error('MySQL Error: ' . mysql_errno());
		$aResultTotal = mysql_fetch_array($rResultTotal);
		$iTotal = $aResultTotal[0];
	
		/*
		 * Output
			 */
			$output = array("sEcho" => intval($_GET['sEcho']), "iTotalRecords" => $iTotal, "iTotalDisplayRecords" => $iFilteredTotal, "aaData" => array());
	
			while ($aRow = mysql_fetch_array($rResult)) {
			$row = array();
			for ($i = 0; $i < count($aColumns); $i++) {
			if ($aColumns[$i] == "version") {
			/* Special output formatting for 'version' column */
			$row[] = ($aRow[$aColumns[$i]] == "0") ? '-' : $aRow[$aColumns[$i]];
			}
			else if ($aColumns[$i] != ' ') {
			/* General output */
				$row[] = $aRow[$aColumns[$i]];
				//$row[] = utf8_encode($aRow[ $aColumns[$i] ]);
			}
			}
			$output['aaData'][] = $row;
	}
	
	return $output;
	}
	}
	
	