<?php
/**
 * Application level View Helper
 *
 * This file is application-wide helper file. You can put all
 * application-wide helper-related methods here.
 *
 * CakePHP(tm) : Rapid Development Framework (http://cakephp.org)
 * Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 *
 * Licensed under The MIT License
 * For full copyright and license information, please see the LICENSE.txt
 * Redistributions of files must retain the above copyright notice.
 *
 * @copyright     Copyright (c) Cake Software Foundation, Inc. (http://cakefoundation.org)
 * @link          http://cakephp.org CakePHP(tm) Project
 * @package       app.View.Helper
 * @since         CakePHP(tm) v 0.2.9
 * @license       http://www.opensource.org/licenses/mit-license.php MIT License
 */

App::uses('HtmlHelper', 'View/Helper');

/**
 * Application helper
 *
 * Add your application-wide methods in the class below, your helpers
 * will inherit them.
 *
 * @package       app.View.Helper
 */
class UtilitiesHelper extends HtmlHelper {
    
    public function getInfluencerLevel($level){
        switch ($level) {
            case Constants::$INFLUENCER_1:
                return "fa fa-volume-off";
                break;
            case Constants::$INFLUENCER_2:
                return "fa fa-volume-down";
                break;
            case Constants::$INFLUENCER_3:
                return "fa fa-volume-medium";
                break;
            case Constants::$INFLUENCER_4:
                return "fa fa-volume-up";
                break;
            case Constants::$INFLUENCER_5:
                return "fa fa-bullhorn";
                break;
            default:
                return "fa fa-volume-off";
                break;
        }
    }
    
    public function getInfluencerLevelProfile($level){
        switch ($level) {
            case Constants::$INFLUENCER_1:
                return "fa fa-volume-off";
                break;
            case Constants::$INFLUENCER_2:
                return "fa fa-volume-down";
                break;
            case Constants::$INFLUENCER_3:
                return "fa fa-volume-medium-profile";
                break;
            case Constants::$INFLUENCER_4:
                return "fa fa-volume-up";
                break;
            case Constants::$INFLUENCER_5:
                return "fa fa-bullhorn";
                break;
            default:
                return "fa fa-volume-off";
                break;
        }
    }
    
    public function getCategoryIcon($category){
        switch ($category) {
            case Constants::$FELICITATION:
                return Constants::$ICON_FELICITATION;
            break;
            case Constants::$THANKS:
                return Constants::$ICON_THANKS;
            break;
            case Constants::$QUESTION:
                return Constants::$ICON_QUESTION;
            break;
            case Constants::$COMMENT:
                return Constants::$ICON_COMMENT;
            break;
            case Constants::$COMPLAINT:
                return Constants::$ICON_COMPLAINT;
            break;
            case Constants::$TROLL:
                return Constants::$ICON_TROLL;
            break;
            default:
                return Constants::$ICON_COMMENT;
            break;
        }
    }
    
    
    public function getCategoryInfo($category){
        switch ($category) {
            case Constants::$FELICITATION:
                return array('Type' => __('Felicitació'),'Icon' => Constants::$ICON_FELICITATION, 'Color' => Constants::$COLOR_FELICITATION);
                break;
            case Constants::$THANKS:
                return array('Type' => __('Agraïment'),'Icon' => Constants::$ICON_THANKS, 'Color' => Constants::$COLOR_THANKS);
                break;
            case Constants::$QUESTION:
                return array('Type' => __('Pregunta'),'Icon' => Constants::$ICON_QUESTION, 'Color' => Constants::$COLOR_QUESTION);
                break;
            case Constants::$COMMENT:
                return array('Type' => __('Comentari'),'Icon' => Constants::$ICON_COMMENT, 'Color' => Constants::$COLOR_COMMENT);
                break;
            case Constants::$COMPLAINT:
                return array('Type' => __('Queixa'),'Icon' => Constants::$ICON_COMPLAINT, 'Color' => Constants::$COLOR_COMPLAINT);
                break;
            case Constants::$TROLL:
                return array('Type' => __('Trolls'),'Icon' => Constants::$ICON_TROLL, 'Color' => Constants::$COLOR_TROLL);
                break;
            default:
                return array('Type' => __('Comentari'),'Icon' => Constants::$ICON_COMMENT, 'Color' => Constants::$COLOR_COMMENT);
                break;
        }
    }
    
    public function getMoodColor($mood){
        if($mood <= Constants::$MOOD_ANGRY){
            $color = Constants::$COLOR_NEGATIVE;
        }else if($mood > Constants::$MOOD_ANGRY && $mood < Constants::$MOOD_HAPPY){
            $color = Constants::$COLOR_NEUTRAL;
        }
        else{
            $color = Constants::$COLOR_POSITIVE; //"#26A4AD";
        }
        
        return $color;
    }
    
    public function getDarkMoodColor($mood){
    	if($mood <= Constants::$MOOD_ANGRY){
    		$color = "#BD4742";
    	}else if($mood > Constants::$MOOD_ANGRY && $mood < Constants::$MOOD_HAPPY){
    		$color = "#6D6D6D";
    	}
    	else{
    		$color = "#227E85";
    	}
    
    	return $color;
    }
    
    
    public function getTrend($trend){
        if($trend <= Constants::$MOOD_ANGRY){
            $result = '<i class="fa fa-arrow-down" style="color:'.Constants::$COLOR_NEGATIVE.';font-size:20px;""></i> &nbsp;'.__('Negativa');
        }else if($trend > Constants::$MOOD_ANGRY && $trend < Constants::$MOOD_HAPPY){
           $result = '<i class="fa fa-arrow-right" style="color:'.Constants::$COLOR_NEUTRAL.';font-size:20px;"></i> &nbsp;'.__('Neutral');
        }
        else{
           $result = '<i class="text-primary fa fa-arrow-up" style="font-size:20px;"></i> &nbsp;'.__('Positiva');
        }
    
        return $result;
    }
    
    
    public function getNumberFollowers($followers){

        if ($followers > 9999 && $followers <= 999999) {
            $followers = round($followers / 1000, 1) . ' K';
        } elseif ($followers > 999999) {
            $followers = round($followers / 1000000, 1) . ' M';
        }
        
        return $followers;
    }
}
